//
//  MainGame.m
//  NewSide
//
//  Created by Bruno Lorandi Pagno on 29/9/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#import "SDLMainGame.h"
#import <iostream>
#include <SDL/SDL.h>

using namespace std;

SDL_Surface *main_screen = NULL;
SDL_Surface *ball;

SDLMainGame::SDLMainGame() {
	// initialization
    cout << "*** Load ***" << endl;
    main_screen = SDL_SetVideoMode(300, 300,32, SDL_SWSURFACE);
    SDL_UnlockSurface(main_screen);
    ball = image_load_from_file("ball.bmp", true);
    x = 20;
    y = 20;
    speed = 4;
    run_game_loop();
}

SDLMainGame::~SDLMainGame() {
	// destruction
}

int SDLMainGame::mainLoop() {
	cout << "*** SDL ***" << endl;
	return 0;
}

void SDLMainGame::handle_keystate(Uint8 *keystate) {
    
    if (keystate[SDLK_UP]) {
        y -= speed;
    }
    
    if (keystate[SDLK_DOWN]) {
        y += speed;
    }
    
    if (keystate[SDLK_LEFT]) {
        x -= speed;
    } 
    
    if (keystate[SDLK_RIGHT]) {
        x += speed;
    }
    
    if (keystate[SDLK_ESCAPE]) {
        quit_the_game();
    }
}

void SDLMainGame::run_game_loop(void) {
    
    Uint32 start_ticks = 0, end_ticks = 0;
    SDL_Event event;
    Uint8 *keystate = NULL;
    
    for (;;) {
        
        start_ticks = SDL_GetTicks();
        
        while (SDL_PollEvent(&event)) {
            
            switch (event.type) {
                    
                case SDL_QUIT:
                    quit_the_game();
            }
        }
        
        keystate = SDL_GetKeyState(NULL);
        
        handle_keystate(keystate);
        update_screen();
        
        end_ticks = SDL_GetTicks();
        cap_fps(start_ticks, end_ticks);
    }
}



void SDLMainGame::cap_fps(Uint32 start_ticks, Uint32 end_ticks) {
    
    int sleep_delay = (1000 / 60) - (end_ticks-start_ticks);
    
    if (sleep_delay > 0) {
        SDL_Delay(sleep_delay);
    }     
}

void SDLMainGame::update_screen(void) {
    
    clear_screen();
    
    blit_surface(ball, main_screen, x, y);
    //draw_scores(lPaddle.score, rPaddle.score);

    SDL_UpdateRect(main_screen, 0, 0, 0, 0);
}

SDL_Surface* SDLMainGame::image_load_from_file(const char *path, int apply_colorkey) {
    SDL_Surface *loaded = SDL_LoadBMP(path);
    if (loaded == NULL) {
        cout << "*** Unable to load image ***" << endl;
        return NULL;
    }
    
    if (apply_colorkey == true) {
        
        SDL_SetColorKey(loaded, SDL_SRCCOLORKEY | SDL_RLEACCEL,
                        (Uint32)SDL_MapRGB(loaded->format, 0, 0, 0));
    }
    
    SDL_Surface *optimized = SDL_DisplayFormat(loaded);
    SDL_FreeSurface(loaded);
    
    return optimized;
}

void SDLMainGame::clear_screen(void) {
    
    SDL_FillRect(main_screen, NULL, SDL_MapRGB(main_screen->format, 0, 0, 0));
}

int SDLMainGame::blit_surface(SDL_Surface *src, SDL_Surface *dest, Sint16 x, Sint16 y) {
    
    SDL_Rect offset;
    offset.x = x;
    offset.y = y;
    
    return SDL_BlitSurface(src, NULL, dest, &offset); 
}

void SDLMainGame::quit_the_game(void) {
    
    SDL_FreeSurface(ball);

    
    SDL_Quit();
    
    exit(0);
}
